@@ -1,9 +1,9 @@
 ;;; sml-mode.el --- Major mode for editing (Standard) ML  -*- lexical-binding: t; coding: utf-8 -*-
 
-;; Copyright (C) 1989-2020  Free Software Foundation, Inc.
+;; Copyright (C) 1989-2023  Free Software Foundation, Inc.
 
 ;; Maintainer: Stefan Monnier <monnier@iro.umontreal.ca>
-;; Version: 6.10
+;; Version: 6.12
 ;; Keywords: SML
 ;; Author:	Lars Bo Nielsen
 ;;		Olin Shivers
@@ -27,7 +27,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -51,11 +51,17 @@
 
 ;;;; News:
 
+;;;;; Changes since 6.11:
+
+;; - Fix indentation of continued strings.
+;; - Use `syntax-propertize' rather than `font-lock-syntactic-keywords'.
+
 ;;;;; Changes since 6.8:
 
 ;; - new var sml-abbrev-skeletons to control whether to include skeletons
 ;;   in the main abbrev table.
 ;; - change parsing rule of "local"
+;; - Improve handling of comments in `sml-cm-mode'.
 
 ;;;;; Changes since 5.0:
 
@@ -167,12 +173,12 @@ notion of \"the end of an outline\".")
 (defvar sml-mode-map
   (let ((map (make-sparse-keymap)))
     ;; Text-formatting commands:
-    (define-key map "\C-c\C-m" 'sml-insert-form)
-    (define-key map "\M-|" 'sml-electric-pipe)
-    (define-key map "\M-\ " 'sml-electric-space)
-    (define-key map [backtab] 'sml-back-to-outer-indent)
+    (define-key map "\C-c\C-m" #'sml-insert-form)
+    (define-key map "\M-|" #'sml-electric-pipe)
+    (define-key map "\M-\ " #'sml-electric-space)
+    (define-key map [backtab] #'sml-back-to-outer-indent)
     ;; The standard binding is C-c C-z, but we add this one for compatibility.
-    (define-key map "\C-c\C-s" 'sml-prog-proc-switch-to)
+    (define-key map "\C-c\C-s" #'sml-prog-proc-switch-to)
     map)
   "The keymap used in `sml-mode'.")
 
@@ -325,7 +331,7 @@ Regexp match data 0 points to the chars."
 
 (defun sml-font-lock-symbols-keywords ()
   (when sml-font-lock-symbols
-    `((,(regexp-opt (mapcar 'car sml-font-lock-symbols-alist) t)
+    `((,(regexp-opt (mapcar #'car sml-font-lock-symbols-alist) t)
        (0 (sml-font-lock-compose-symbol))))))
 
 ;; The font lock regular expressions.
@@ -334,25 +340,25 @@ Regexp match data 0 points to the chars."
   `(;;(sml-font-comments-and-strings)
     (,(concat "\\_<\\(fun\\|and\\)\\s-+" sml-tyvarseq-re
               "\\(" sml-id-re "\\)\\s-+[^ \t\n=]")
-     (1 font-lock-keyword-face)
-     (2 font-lock-function-name-face))
+     (1 'font-lock-keyword-face)
+     (2 'font-lock-function-name-face))
     (,(concat "\\_<\\(\\(?:data\\|abs\\|with\\|eq\\)?type\\)\\s-+"
               sml-tyvarseq-re "\\(" sml-id-re "\\)")
-     (1 font-lock-keyword-face)
-     (2 font-lock-type-def-face))
+     (1 'font-lock-keyword-face)
+     (2 'font-lock-type-def-face))
     (,(concat "\\_<\\(val\\)\\s-+\\(?:" sml-id-re "\\_>\\s-*\\)?\\("
               sml-id-re "\\)\\s-*[=:]")
-     (1 font-lock-keyword-face)
-     (2 font-lock-variable-name-face))
+     (1 'font-lock-keyword-face)
+     (2 'font-lock-variable-name-face))
     (,(concat "\\_<\\(structure\\|functor\\|abstraction\\)\\s-+\\("
               sml-id-re "\\)")
-     (1 font-lock-keyword-face)
-     (2 font-lock-module-def-face))
+     (1 'font-lock-keyword-face)
+     (2 'font-lock-module-def-face))
     (,(concat "\\_<\\(signature\\)\\s-+\\(" sml-id-re "\\)")
-     (1 font-lock-keyword-face)
-     (2 font-lock-interface-def-face))
+     (1 'font-lock-keyword-face)
+     (2 'font-lock-interface-def-face))
 
-    (,sml-keywords-regexp . font-lock-keyword-face)
+    (,sml-keywords-regexp (0 'font-lock-keyword-face))
     ,@(sml-font-lock-symbols-keywords))
   "Regexps matching standard SML keywords.")
 
@@ -388,12 +394,12 @@ Regexp match data 0 points to the chars."
     st)
   "Syntax table for text-properties.")
 
-(defconst sml-font-lock-syntactic-keywords
-  `(("^\\s-*\\(\\\\\\)" (1 ',sml-syntax-prop-table))))
+(defconst sml-syntax-propertize
+  (syntax-propertize-rules
+   ("^\\s-*\\(\\\\\\)" (1 sml-syntax-prop-table))))
 
 (defconst sml-font-lock-defaults
-  '(sml-font-lock-keywords nil nil nil nil
-    (font-lock-syntactic-keywords . sml-font-lock-syntactic-keywords)))
+  '(sml-font-lock-keywords nil nil nil nil))
 
 
 ;;; Indentation with SMIE
@@ -718,6 +724,19 @@ Takes the form (DIR POS . TOKENS).")
 		  alist)))))
     alist))
 
+(defun sml-smie-indent-string-continuation ()
+  (let ((ppss (syntax-ppss)))
+    (when (and (nth 3 ppss)      ;Inside a string.
+               (eq ?\\ (char-after))
+               (save-excursion (skip-chars-backward " \t") (bolp)))
+      (save-excursion
+        (let ((start (nth 8 ppss)))
+          (forward-line -1)
+          (if (< (point) start)
+              (goto-char start)
+            (skip-chars-forward " \t"))
+          (current-column))))))
+
 ;;; Generic prog-proc interaction.
 
 (require 'comint)
@@ -725,11 +744,11 @@ Takes the form (DIR POS . TOKENS).")
 
 (defvar sml-prog-proc-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map [?\C-c ?\C-l] 'sml-prog-proc-load-file)
-    (define-key map [?\C-c ?\C-c] 'sml-prog-proc-compile)
-    (define-key map [?\C-c ?\C-z] 'sml-prog-proc-switch-to)
-    (define-key map [?\C-c ?\C-r] 'sml-prog-proc-send-region)
-    (define-key map [?\C-c ?\C-b] 'sml-prog-proc-send-buffer)
+    (define-key map [?\C-c ?\C-l] #'sml-prog-proc-load-file)
+    (define-key map [?\C-c ?\C-c] #'sml-prog-proc-compile)
+    (define-key map [?\C-c ?\C-z] #'sml-prog-proc-switch-to)
+    (define-key map [?\C-c ?\C-r] #'sml-prog-proc-send-region)
+    (define-key map [?\C-c ?\C-b] #'sml-prog-proc-send-buffer)
     ;; FIXME: Add
     ;; (define-key map [?\M-C-x] 'sml-prog-proc-send-defun)
     ;; (define-key map [?\C-x ?\C-e] 'sml-prog-proc-send-last-sexp)
@@ -870,7 +889,7 @@ AND-GO if non-nil indicate to additionally switch to the process's buffer."
 
 (defvar sml-prog-proc-comint-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-l" 'sml-prog-proc-load-file)
+    (define-key map "\C-c\C-l" #'sml-prog-proc-load-file)
     map))
 
 (define-derived-mode sml-prog-proc-comint-mode comint-mode "Sml-Prog-Proc-Comint"
@@ -995,7 +1014,8 @@ Prefix arg AND-GO also means to switch to the read-eval-loop buffer afterwards."
   "Commands used by default by `sml-sml-prog-proc-compile'.
 Each command is associated with its \"main\" file.
 It is perfectly OK to associate several files with a command or several
-commands with the same file.")
+commands with the same file."
+  :type '(alist string string))
 
 ;; FIXME: Try to auto-detect the process and set those vars accordingly.
 
@@ -1092,7 +1112,7 @@ See `compilation-error-regexp-alist' for a description of the format.")
      sml-host-name)))
 
 ;;;###autoload
-(defalias 'run-sml 'sml-run)
+(defalias 'run-sml #'sml-run)
 
 ;;;###autoload
 (defun sml-run (cmd arg &optional host)
@@ -1127,7 +1147,7 @@ on which to run CMD using `remote-shell-program'.
                            ;; first look relative to the current directory.
                            ;; Emacs-21 does it for us, but not Emacs-20.
                            (cons default-directory exec-path) exec-path)))
-        (pop-to-buffer (apply 'make-comint pname cmd file args)))
+        (pop-to-buffer (apply #'make-comint pname cmd file args)))
 
       (inferior-sml-mode)
       (goto-char (point-max))
@@ -1145,8 +1165,8 @@ With a prefix argument AND-GO switch to the repl buffer as well."
 (defvar inferior-sml-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map comint-mode-map)
-    (define-key map "\C-c\C-s" 'sml-run)
-    (define-key map "\t" 'completion-at-point)
+    (define-key map "\C-c\C-s" #'sml-run)
+    (define-key map "\t" #'completion-at-point)
     map)
   "Keymap for inferior-sml mode.")
 
@@ -1198,8 +1218,12 @@ With a prefix argument AND-GO switch to the repl buffer as well."
                    b1 t)
               (setq e1 (match-beginning 0))
               (setq b2 (match-end 0))
-              (smerge-refine-subst b1 e1 b2 e2
-                                   '((face . smerge-refined-change))))))))))
+              (funcall
+               (if (fboundp 'smerge-refine-regions) ;Emacs≥26
+                   #'smerge-refine-regions
+                 'smerge-refine-subst)
+               b1 e1 b2 e2
+               '((face . smerge-refined-change))))))))))
 
 (define-derived-mode inferior-sml-mode sml-prog-proc-comint-mode "Inferior-SML"
   "Major mode for interacting with an inferior SML process.
@@ -1249,7 +1273,7 @@ TAB file name completion, as in shell-mode, etc.."
   (sml-mode-variables)
 
   ;; We have to install it globally, 'cause it's run in the *source* buffer :-(
-  (add-hook 'next-error-hook 'inferior-sml-next-error-hook)
+  (add-hook 'next-error-hook #'inferior-sml-next-error-hook)
 
   ;; Make TAB add a " rather than a space at the end of a file name.
   (setq-local comint-completion-addsuffix '("/" . "\""))
@@ -1301,6 +1325,7 @@ See also (info \"(sml-mode)Top\")."
   (setq-local electric-indent-chars
               (cons ?\; (if (boundp 'electric-indent-chars)
                             electric-indent-chars '(?\n))))
+  (setq-local syntax-propertize-function sml-syntax-propertize)
   (setq-local electric-layout-rules
               `((?\; . ,(lambda ()
                           (save-excursion
@@ -1321,6 +1346,7 @@ See also (info \"(sml-mode)Top\")."
   (smie-setup sml-smie-grammar #'sml-smie-rules
               :backward-token #'sml-smie-backward-token
               :forward-token #'sml-smie-forward-token)
+  (add-hook 'smie-indent-functions #'sml-smie-indent-string-continuation nil t)
   (setq-local parse-sexp-ignore-comments t)
   (setq-local comment-start "(* ")
   (setq-local comment-end " *)")
@@ -1589,11 +1615,11 @@ the corresponding form is inserted."
 (defun sml-insert-form (name newline)
   "Interactive short-cut to insert the NAME common SML form.
 If a prefix argument is given insert a NEWLINE and indent first, or
-just move to the proper indentation if the line is blank\; otherwise
+just move to the proper indentation if the line is blank; otherwise
 insert at point (which forces indentation to current column).
 
 The default form to insert is whatever you inserted last time
-\(just hit return when prompted\)\; otherwise the command reads with
+\(just hit return when prompted); otherwise the command reads with
 completion from `sml-forms-alist'."
   (interactive
    (list (completing-read
@@ -1744,7 +1770,13 @@ MAINFILE is the top level file of the project."
 (define-derived-mode sml-cm-mode fundamental-mode "SML-CM"
   "Major mode for SML/NJ's Compilation Manager configuration files."
   (setq-local sml-prog-proc-descriptor sml-pp-functions)
-  (setq-local font-lock-defaults '(sml-cm-font-lock-keywords nil t nil nil)))
+  (setq-local font-lock-defaults '(sml-cm-font-lock-keywords nil t nil nil))
+  (setq-local comment-start "(* ")
+  (setq-local comment-end " *)")
+  (setq-local comment-start-skip "(\\*+\\s-*")
+  (setq-local comment-end-skip "\\s-*\\*+)")
+  ;; No need to quote nested comments markers.
+  (setq-local comment-quote-nested nil))
 
 ;;;
 ;;; ML-Lex support
@@ -1849,137 +1881,10 @@ If nil, align it with previous cases."
 ;;;###autoload
 (define-derived-mode sml-yacc-mode sml-mode "SML-Yacc"
   "Major Mode for editing ML-Yacc files."
-  (setq-local indent-line-function 'sml-yacc-indent-line)
+  (setq-local indent-line-function #'sml-yacc-indent-line)
   (setq-local font-lock-defaults sml-yacc-font-lock-defaults))
 
 
-
-;;;; ChangeLog:
-
-;; 2020-02-10  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el: Parse "local" correctly
-;; 
-;; 	Also use `setq-local` and `defvar-local`.
-;; 
-;; 	(sml-smie--pending-token): New var.
-;; 	(sml-smie-forward-token, sml-smie-backward-token): Use it for "local".
-;; 	(sml-smie-rules): Drop ad-hoc indentation rule for "local".
-;; 
-;; 2018-10-08  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el: New var sml-abbrev-skeletons; bump vers to 6.9
-;; 
-;; 2017-12-12  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	Bump version to make new release
-;; 
-;; 2016-10-26  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el (sml-smie-rules): Remove incoherent
-;; 	rule.
-;; 
-;; 2016-08-14  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el (sml-tyvarseq-re): Backtrack less
-;; 	(bug#24205)
-;; 
-;; 2016-08-04  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode/sml-mode.el: Use cl-lib.
-;; 
-;; 2016-07-11  Paul Eggert	 <eggert@cs.ucla.edu>
-;; 
-;; 	Fix some quoting problems in doc strings
-;; 
-;; 	Most of these are minor issues involving, e.g., quoting `like this' 
-;; 	instead of 'like this'.	 A few involve escaping ` and ' with a preceding
-;; 	\= when the characters should not be turned into curved single quotes.
-;; 
-;; 2015-02-12  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode/sml-mode.el: Bump version to make a new release
-;; 
-;; 2015-02-12  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode/sml-mode.el (sml-smie-rules): Work around Emacs-24.3 bug.
-;; 
-;; 2015-02-09  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el: Add prettify-symbols-mode and Poly/ML support
-;; 
-;; 	(sml-font-lock-symbols): Mark obsolete.
-;; 	(sml-font-lock-compose-symbol): Use symbol-boundaries rather 
-;; 	word-boundaries.
-;; 	(sml-smie-rules): Tweak indentation after "fun".
-;; 	(sml-smie-definitional-equal-p): Handle ":>".
-;; 	(sml-prog-proc-send-string): Try and avoid mixing prompt/userinput and 
-;; 	process output on the same line.
-;; 	(sml-error-regexp-alist): Add Poly/ML support.
-;; 	(sml-mode): Add prettify-symbols-mode support.
-;; 
-;; 2014-10-15  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el (sml-smie-grammar): Add "withtype".
-;; 	(sml-smie-rules): Use pcase.
-;; 	(sml-smie-non-nested-of-p): Rewrite to avoid regexp and stay closer to
-;; 	point.
-;; 
-;; 2014-06-17  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode/sml-mode.el: Release new version.
-;; 
-;; 2014-06-04  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el (sml--rightalign-and-p): New function.
-;; 
-;; 2014-06-03  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode/sml-mode.el (sml-smie-grammar): Add include&sharing.
-;; 	(sml-smie-rules): Add functor rule.
-;; 	(sml-smie-definitional-equal-p): Use smie-backward-sexp.
-;; 
-;; 2013-12-27  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el (sml-smie-rules): Adjust for new :close-all rule.
-;; 
-;; 2013-03-03  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el (sml-imenu-regexp): Make it a const.
-;; 	(sml-imenu-create-index): Don't assume we'll find an = after structure.
-;; 
-;; 2013-01-24  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode: Release 6.3, bugfixes
-;; 
-;; 2013-01-24  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el (sml-mode-variables): Set sml-prog-proc-descriptor here...
-;; 	(sml-mode): ... instead of here.
-;; 
-;; 2013-01-24  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el (inferior-sml-mode): Fix comint-completion-addsuffix.
-;; 
-;; 2012-12-04  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* packages/sml-mode/sml-mode.el (sml-mode-map): Add back C-c C-s
-;; 	binding.
-;; 	(sml-prog-proc-switch-to): Add missing interactive spec.
-;; 
-;; 2012-10-31  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	* sml-mode.el: Integrate BUGS&NEWS; re-add run-sml.
-;; 
-;; 2012-10-22  Stefan Monnier  <monnier@iro.umontreal.ca>
-;; 
-;; 	Add SML-mode.
-;; 
-;; 	git-subtree-dir: packages/sml-mode git-subtree-mainline:
-;; 	a50c62a632aa422957a0e4224d5c6e4c2366156b git-subtree-split:
-;; 	d2ef91a138487b521bb7e08722b8d122aa757b5a
-;; 
-
 (provide 'sml-mode)
 
 ;;; sml-mode.el ends here
